//
//  DPCurrencyManager.m
//  Currency
//
//  Created by Hjalti Jakobsson on 17.8.2010.
//  Copyright 2010 Hjalti Jakobsson. All rights reserved.
//

#import "DPCurrencyManager.h"
#import "DPCurrencyHandler.h"
#import "DPCurrency.h"
#import "Currency.h"
#import "Record.h"

@interface DPCurrencyManager()
- (void)updateDataStore:(NSArray*)currencies;
- (Currency*)insertCurrencyFromDPCurrency:(DPCurrency*)currency;
- (Currency*)currencyForDPCurrency:(DPCurrency*)currency;
- (BOOL)hasTodaysRecord;
@end


@implementation DPCurrencyManager
@synthesize managedObjectContext, delegate;

- (id)initWithManagedContext:(NSManagedObjectContext*)aContext usingDelegate:(id)aDelegate
{
	if(self = [super init])
	{
		self.managedObjectContext = aContext;
		self.delegate = aDelegate;
		currencyHandler = [[DPCurrencyHandler alloc] init];
		[currencyHandler setDelegate:self];
		
		return self;
	}
	
	return nil;
}

- (id)initWithManagedContext:(NSManagedObjectContext*)aContext
{
	return [self initWithManagedContext:aContext usingDelegate:nil];
}

- (id)init
{
	return [self initWithManagedContext:nil];
}

- (BOOL)updateData
{
	if(![currencyHandler isRefreshing])
	{
		[currencyHandler update];
		
		return YES;
	}
	
	return NO;
}

- (void)updateDataStore:(NSArray*)currencies
{
	if([currencies count] == 0)
		return;
	
	//Skip update if we already have updated today
	if([self hasTodaysRecord])
		return;
	
	if(!self.managedObjectContext)
	{
		NSLog(@"Error: No context set");
		return;
	}
	
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Record" inManagedObjectContext:self.managedObjectContext];
	
	for(DPCurrency *currency in currencies)
	{		
		Record *record = (Record*)[NSEntityDescription insertNewObjectForEntityForName:[entity name] inManagedObjectContext:self.managedObjectContext];
		record.date = [NSDate date];
		record.buy = [NSNumber numberWithFloat:currency.buy];
		record.sell = [NSNumber numberWithFloat:currency.sell];
		
		Currency *storedCurrency = [self currencyForDPCurrency:currency];
		
		record.currency = storedCurrency;
	}
	
	NSError *error = nil;
	
	if(![self.managedObjectContext save:&error])
	{
		NSLog(@"Unresolved error %@, %@", error, [error localizedDescription]);
	}
}



- (BOOL)hasTodaysRecord
{
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Record" inManagedObjectContext:self.managedObjectContext];
	
	NSFetchRequest *request = [[[NSFetchRequest alloc] init] autorelease];
	[request setEntity:entity];
	
	//Check if we can find any record from today
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"date > %@", [NSDate dateWithTimeIntervalSinceNow:-86400]]; //One day in seconds
	[request setPredicate:predicate];
	
	return ([self.managedObjectContext countForFetchRequest:request error:nil]);
}

- (Currency*)currencyForDPCurrency:(DPCurrency*)currency
{
	NSEntityDescription *entity = [NSEntityDescription entityForName:@"Currency" inManagedObjectContext:self.managedObjectContext];
	
	NSFetchRequest *request = [[NSFetchRequest alloc] init];
	[request setEntity:entity];
	
	//We use the identifier as a key for our search, we should never have more than one record per currency/identifier
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"identifier == %@", currency.identifier];
	[request setPredicate:predicate];
	
	NSError *error = nil;
	
	NSArray *results = [self.managedObjectContext executeFetchRequest:request error:&error];
	[request release];
	
	if(error)
	{
		NSLog(@"Unresolved error %@, %@", error, [error localizedDescription]);
		return nil;
	}	
	
	if([results count] > 0)
	{
		//We should only have one match so we return the first object in the result set
		return [results objectAtIndex:0];
	}
	else
	{
		//If item is not found we need to create it and return it
		return [self insertCurrencyFromDPCurrency:currency];
	}
	
	return nil;
}

- (Currency*)insertCurrencyFromDPCurrency:(DPCurrency*)currency
{
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Currency" inManagedObjectContext:self.managedObjectContext];
	Currency *newCurrency = (Currency*)[NSEntityDescription insertNewObjectForEntityForName:[entity name] inManagedObjectContext:self.managedObjectContext];

	newCurrency.name = currency.name;
	newCurrency.identifier = currency.identifier;
	newCurrency.iconURL = [currency.iconURL absoluteString];
	
	NSError *error = nil;
	
	if(![self.managedObjectContext save:&error])
	{
		NSLog(@"Unresolved error %@, %@", error, [error localizedDescription]);
	}
	
	return newCurrency;
}

#pragma mark -
#pragma mark Currency Handler Delegates

- (void)updateCompletedWithResults:(NSArray*)results
{
	[self updateDataStore:results];
	
	if([delegate respondsToSelector:@selector(updateCompleted)])
		[delegate performSelector:@selector(updateCompleted)];
}

- (void)updateFailedWithError:(NSError*)error
{
	if([delegate respondsToSelector:@selector(updateFailedWithError:)])
		[delegate performSelector:@selector(updateFailedWithError:) withObject:error];
}

#pragma mark -
#pragma mark Clean Up

- (void)dealloc
{
	[currencyHandler release];
	[super dealloc];
}

@end
