//
//  DPLineChartView.m
//  Currency
//
//  Created by Hjalti Jakobsson on 17.8.2010.
//  Copyright 2010 Hjalti Jakobsson. All rights reserved.
//

#import "DPLineChartView.h"

#define kLabelRightMargin 10.0
#define kChartBottomOffset 20.0
#define kHeaderTopOffset 15.0
#define kLabelWidth 55.0
#define kCornerRadius 5.0
#define kHeaderHeight 80.0

@interface DPLineChartView()
- (void)drawYLabelsInContext:(CGContextRef)context;
- (void)drawDataPointsInContext:(CGContextRef)context;
- (void)drawHeaderInContext:(CGContextRef)context;
- (void)drawRoundRectInContext:(CGContextRef)context;
@end

@implementation DPLineChartView
@synthesize delegate, lineColor, gridColor, headerString, roundedCorners, labelFont, headerFont;

- (id)initWithCoder:(NSCoder *)aDecoder
{
	if(self = [super initWithCoder:aDecoder])
	{
		roundedCorners = YES;
		
		if(roundedCorners)
			self.backgroundColor = [UIColor blackColor];
		
		self.headerFont = [UIFont boldSystemFontOfSize:15.0];
		self.labelFont = [UIFont boldSystemFontOfSize:13.0];
		normalizedData = [[NSMutableArray alloc] init];
		paths = [[NSMutableArray alloc] init];
		self.gridColor = [UIColor grayColor];
		self.lineColor = [UIColor colorWithRed:56.0/255.0 green:120.0/255.0 blue:199.0/255.0 alpha:1.0];
		
		return self;
    }
	
    return nil;
}

- (void)setRoundedCorners:(BOOL)flag
{
	if(flag)
		self.backgroundColor = [UIColor blackColor];

	roundedCorners = flag;
	[self setNeedsDisplay];
}

- (void)setLineColor:(UIColor*)color
{
	if(lineColor == color)
		return;
	
	[lineColor release];
	
	lineColor = [color retain];
	[self setNeedsDisplay];
}

- (void)setGridColor:(UIColor*)color
{
	if(gridColor == color)
		return;
	
	[gridColor release];
	
	gridColor = [color retain];
	[self setNeedsDisplay];
}

/* Asks the datasource for the data, finds the highest/lowest values and normalizes the data */

- (void)reloadData
{
	if(self.delegate && [delegate conformsToProtocol:@protocol(DPLineChartViewDataSource)])
	{
		if([delegate respondsToSelector:@selector(headerForLineChartView:)])
			self.headerString = [delegate performSelector:@selector(headerForLineChartView:) withObject:self];
		
		[normalizedData removeAllObjects];
		
		NSInteger numPoints = [delegate numberOfPointsInLineChartView:self];
		
		if(numPoints)
		{
			maxValue = 0.0;
			minValue = FLT_MAX;
			
			NSMutableArray *allPoints = [[NSMutableArray alloc] initWithCapacity:numPoints];
			
			NSUInteger pointNum;
			
			for(pointNum = 0; pointNum < numPoints; pointNum++)
			{
				float pointValue = [delegate lineChartView:self pointValueAtIndex:pointNum];
				
				maxValue = MAX(pointValue, maxValue);
				minValue = MIN(pointValue, minValue);
				
				[allPoints addObject:[NSNumber numberWithFloat:pointValue]];
			}
			
			for (NSNumber *dataValue in allPoints)
			{
				float delta;
				
				if((maxValue - minValue) <= 0)
					delta = 1.0;
				else
					delta = maxValue - minValue;
				
				NSNumber *value = [[NSNumber alloc] initWithFloat:([dataValue floatValue] - minValue) / delta];
				[normalizedData addObject:value];
				[value release];
			}
			
			[allPoints release];
		}
	}
}

/* Fills the bounds with rounded white rectangle to simulate rounded corners on the view */

- (void)drawRoundRectInContext:(CGContextRef)context
{
	CGContextSaveGState(context);
	
	CGRect rrect = self.bounds;
    rrect.size.height = rrect.size.height - 1.0;
    rrect.size.width = rrect.size.width - 1.0;
    rrect.origin.x = rrect.origin.x + (1.0 / 2);
    rrect.origin.y = rrect.origin.y + (1.0 / 2);
	
    CGFloat radius = kCornerRadius;
    CGFloat minx = CGRectGetMinX(rrect);
    CGFloat midx = CGRectGetMidX(rrect);
    CGFloat maxx = CGRectGetMaxX(rrect);
    CGFloat miny = CGRectGetMinY(rrect);
    CGFloat midy = CGRectGetMidY(rrect);
    CGFloat maxy = CGRectGetMaxY(rrect);	
	
    CGContextSetRGBFillColor(context, 1.0, 1.0, 1.0, 1.0);
    CGContextMoveToPoint(context, minx, midy);
    CGContextAddArcToPoint(context, minx, miny, midx, miny, radius);
    CGContextAddArcToPoint(context, maxx, miny, maxx, midy, radius);
    CGContextAddArcToPoint(context, maxx, maxy, midx, maxy, radius);
    CGContextAddArcToPoint(context, minx, maxy, minx, midy, radius);
    CGContextClosePath(context);
    CGContextDrawPath(context, kCGPathFill);
	
	CGContextRestoreGState(context);
}

/* Does all the drawing */

- (void)drawRect:(CGRect)rect
{
	if ([normalizedData count] < 1)
		return;
	
	CGContextRef context = UIGraphicsGetCurrentContext();
	
	if(roundedCorners)
		[self drawRoundRectInContext:context];
	
	[self drawDataPointsInContext:context];
	[self drawYLabelsInContext:context];
	[self drawHeaderInContext:context];

}

/* Draws the lines between the data points */

- (void)drawDataPointsInContext:(CGContextRef)context
{
	CGContextSaveGState(context);
	
	chartSize = self.bounds.size;
	chartSize.width -= kLabelWidth+kLabelRightMargin;
	chartSize.height -= kHeaderHeight;
	
	CGFloat offset = (self.bounds.size.height-chartSize.height-kChartBottomOffset);
	
	CGFloat maxX = chartSize.width;
	CGFloat maxY = chartSize.height;
	
	CGContextSetLineWidth(context, 2);
	CGContextSetLineJoin(context, kCGLineJoinBevel);
	
	CGColorRef strokeColor = [lineColor CGColor];
	CGContextSetStrokeColorWithColor(context, strokeColor);
	
	CGContextBeginPath(context);				
	CGContextMoveToPoint(context, 0.0, (maxY - (maxY * [[normalizedData objectAtIndex:0] floatValue]))+offset);
	
	for(int i = 1; i < [normalizedData count]; i++)
	{
		CGContextAddLineToPoint(context, maxX * ((CGFloat)i / ([normalizedData count] - 1)), 
										(maxY - (maxY * [[normalizedData objectAtIndex:i] floatValue]))+offset);
	}
	
	CGContextStrokePath(context);
	CGContextRestoreGState(context);
}

- (void)drawYLabelsInContext:(CGContextRef)context
{
	CGContextSaveGState(context);
	
	float range = maxValue - minValue;	
	float step = range / 5.0;
	
	CGFloat labelHeight = 20.0;	
	CGFloat dash[] = {4.0, 4.0}; 

	for(int i = 0; i < 6.0; i++)
	{
		CGFloat yPos = i*(chartSize.height/5.0);
		
		//Draw grid lines
		
		CGContextSetLineDash(context, 0.0, dash, 1.0);	
		CGContextSetLineWidth(context, 0.2f);
		
		CGPoint startPoint = CGPointMake(0, (self.bounds.size.height - yPos)-kChartBottomOffset);
		CGPoint endPoint = CGPointMake(self.bounds.size.width, (self.bounds.size.height - yPos)-kChartBottomOffset);
		
		CGContextMoveToPoint(context, startPoint.x, startPoint.y);
		CGContextAddLineToPoint(context, endPoint.x - (kLabelWidth + kLabelRightMargin), endPoint.y);
		CGContextClosePath(context);
		
		CGContextSetStrokeColorWithColor(context, gridColor.CGColor);
		CGContextStrokePath(context);
		
		//Draw line values
		CGRect valueStringRect = CGRectMake(self.bounds.size.width-kLabelWidth-kLabelRightMargin, ((self.bounds.size.height - yPos) - labelHeight/2.0)-kChartBottomOffset, kLabelWidth, labelHeight);
		NSString *label = [NSString stringWithFormat:@"%1.2f", minValue+(step*i)];
		[[UIColor darkGrayColor] set];
		[label drawInRect:valueStringRect withFont:labelFont lineBreakMode:UILineBreakModeTailTruncation alignment:UITextAlignmentRight];
	}
	
	CGContextRestoreGState(context);
}

- (void)drawHeaderInContext:(CGContextRef)context
{
	if(headerString)
	{	
		CGContextSaveGState(context);
		
		[[UIColor darkGrayColor] set];
		CGRect headerRect = {0.0, kHeaderTopOffset, self.bounds.size.width, 20.0};
		[headerString drawInRect:headerRect withFont:headerFont lineBreakMode:UILineBreakModeTailTruncation alignment:UITextAlignmentCenter];
	
		CGContextRestoreGState(context);
	}
}

- (void)dealloc
{
	[headerFont release];
	[labelFont release];
	[headerString release];
	[gridColor release];
	[lineColor release];
	[normalizedData release];
    [super dealloc];
}


@end
